@@ -1,6 +1,6 @@
 /*  bamtk.c -- main samtools command front-end.
 
-    Copyright (C) 2008-2019 Genome Research Ltd.
+    Copyright (C) 2008-2020 Genome Research Ltd.
 
     Author: Heng Li <lh3@sanger.ac.uk>
 
@@ -46,7 +46,6 @@ int bam_fillmd(int argc, char *argv[]);
 int bam_idxstats(int argc, char *argv[]);
 int bam_markdup(int argc, char *argv[]);
 int main_samview(int argc, char *argv[]);
-int main_import(int argc, char *argv[]);
 int main_reheader(int argc, char *argv[]);
 int main_cut_target(int argc, char *argv[]);
 int main_phase(int argc, char *argv[]);
@@ -65,6 +64,8 @@ int main_addreplacerg(int argc, char *argv[]);
 int faidx_main(int argc, char *argv[]);
 int dict_main(int argc, char *argv[]);
 int fqidx_main(int argc, char *argv[]);
+int amplicon_clip_main(int argc, char *argv[]);
+int main_ampliconstats(int argc, char *argv[]);
 
 const char *samtools_version()
 {
@@ -96,6 +97,7 @@ static void usage(FILE *fp)
 "     targetcut      cut fosmid regions (for fosmid pool only)\n"
 "     addreplacerg   adds or replaces RG tags\n"
 "     markdup        mark duplicates\n"
+"     ampliconclip   clip oligos from the end of reads\n"
 "\n"
 "  -- File operations\n"
 "     collate        shuffle and group alignments by name\n"
@@ -116,6 +118,7 @@ static void usage(FILE *fp)
 "     idxstats       BAM index stats\n"
 "     phase          phase heterozygotes\n"
 "     stats          generate stats (former bamcheck)\n"
+"     ampliconstats  generate amplicon specific stats\n"
 "\n"
 "  -- Viewing\n"
 "     flags          explain BAM flags\n"
@@ -123,12 +126,6 @@ static void usage(FILE *fp)
 "     view           SAM<->BAM<->CRAM conversion\n"
 "     depad          convert padded BAM to unpadded BAM\n"
 "\n");
-#ifdef _WIN32
-    fprintf(fp,
-"Note: The Windows version of SAMtools is mainly designed for read-only\n"
-"      operations, such as viewing the alignments and generating the pileup.\n"
-"      Binary files generated by the Windows version may be buggy.\n\n");
-#endif
 }
 
 // This is a tricky one, but on Windows the filename wildcard expansion is done by
@@ -143,6 +140,11 @@ static void usage(FILE *fp)
 int _CRT_glob = 0;
 #endif
 
+static void bam_import_err(void) {
+    fprintf(stderr, "[main] \"samtools import\" has been removed. "
+            "Please use \"samtools view\" instead.\n");
+}
+
 int main(int argc, char *argv[])
 {
 #ifdef _WIN32
@@ -163,7 +165,7 @@ int main(int argc, char *argv[])
 
     int ret = 0;
     if (strcmp(argv[1], "view") == 0)           ret = main_samview(argc-1, argv+1);
-    else if (strcmp(argv[1], "import") == 0)    ret = main_import(argc-1, argv+1);
+    else if (strcmp(argv[1], "import") == 0)    { bam_import_err(); return 1; }
     else if (strcmp(argv[1], "mpileup") == 0)   ret = bam_mpileup(argc-1, argv+1);
     else if (strcmp(argv[1], "merge") == 0)     ret = bam_merge(argc-1, argv+1);
     else if (strcmp(argv[1], "sort") == 0)      ret = bam_sort(argc-1, argv+1);
@@ -176,6 +178,7 @@ int main(int argc, char *argv[])
     else if (strcmp(argv[1], "fixmate") == 0)   ret = bam_mating(argc-1, argv+1);
     else if (strcmp(argv[1], "rmdup") == 0)     ret = bam_rmdup(argc-1, argv+1);
     else if (strcmp(argv[1], "markdup") == 0)   ret = bam_markdup(argc-1, argv+1);
+    else if (strcmp(argv[1], "ampliconclip") == 0) ret = amplicon_clip_main(argc-1, argv+1);
     else if (strcmp(argv[1], "flagstat") == 0 ||
              strcmp(argv[1], "flagstats") == 0) ret = bam_flagstat(argc-1, argv+1);
     else if (strcmp(argv[1], "calmd") == 0)     ret = bam_fillmd(argc-1, argv+1);
@@ -206,11 +209,12 @@ int main(int argc, char *argv[])
         return 1;
     }
     else if (strcmp(argv[1], "tview") == 0)   ret = bam_tview_main(argc-1, argv+1);
+    else if (strcmp(argv[1], "ampliconstats") == 0)     ret = main_ampliconstats(argc-1, argv+1);
     else if (strcmp(argv[1], "--version") == 0) {
         printf(
 "samtools %s\n"
 "Using htslib %s\n"
-"Copyright (C) 2019 Genome Research Ltd.\n",
+"Copyright (C) 2020 Genome Research Ltd.\n",
                samtools_version(), hts_version());
     }
     else if (strcmp(argv[1], "--version-only") == 0) {
