@@ -1,7 +1,7 @@
 /*  phase.c -- phase subcommand.
 
     Copyright (C) 2011 Broad Institute.
-    Copyright (C) 2013-2016 Genome Research Ltd.
+    Copyright (C) 2013-2016, 2019 Genome Research Ltd.
 
     Author: Heng Li <lh3@sanger.ac.uk>
 
@@ -36,6 +36,7 @@ DEALINGS IN THE SOFTWARE.  */
 #include "htslib/kstring.h"
 #include "sam_opts.h"
 #include "samtools.h"
+#include "htslib/hts_os.h"
 
 #include "htslib/kseq.h"
 KSTREAM_INIT(gzFile, gzread, 16384)
@@ -51,15 +52,15 @@ KSTREAM_INIT(gzFile, gzread, 16384)
 
 typedef struct {
     // configurations, initialized in the main function
-    int flag, k, min_baseQ, min_varLOD, max_depth;
+    int flag, k, min_baseQ, min_varLOD, max_depth, no_pg;
     // other global variables
     int vpos_shift;
     samFile* fp;
-    bam_hdr_t* fp_hdr;
-    char *pre;
+    sam_hdr_t* fp_hdr;
+    char *pre, *arg_list;
     char *out_name[3];
     samFile* out[3];
-    bam_hdr_t* out_hdr[3];
+    sam_hdr_t* out_hdr[3];
     // alignment queue
     int n, m;
     bam1_t **b;
@@ -502,7 +503,7 @@ static int readaln(void *data, bam1_t *b)
     return ret;
 }
 
-static khash_t(set64) *loadpos(const char *fn, bam_hdr_t *h)
+static khash_t(set64) *loadpos(const char *fn, sam_hdr_t *h)
 {
     gzFile fp;
     kstream_t *ks;
@@ -510,9 +511,15 @@ static khash_t(set64) *loadpos(const char *fn, bam_hdr_t *h)
     kstring_t *str;
     khash_t(set64) *hash;
 
+    fp = strcmp(fn, "-")? gzopen(fn, "r") : gzdopen(fileno(stdin), "r");
+    if (fp == NULL) {
+        print_error_errno("phase", "Couldn't open site file '%s'", fn);
+        return NULL;
+    }
+
     hash = kh_init(set64);
     str = calloc(1, sizeof(kstring_t));
-    fp = strcmp(fn, "-")? gzopen(fn, "r") : gzdopen(fileno(stdin), "r");
+
     ks = ks_init(fp);
     while (ks_getuntil(ks, 0, str, &dret) >= 0) {
         int tid = bam_name2id(h, str->s);
@@ -556,7 +563,15 @@ static int start_output(phaseg_t *g, int c, const char *middle, const htsFormat
         return -1;
     }
 
-    g->out_hdr[c] = bam_hdr_dup(g->fp_hdr);
+    g->out_hdr[c] = sam_hdr_dup(g->fp_hdr);
+    if (!g->no_pg && sam_hdr_add_pg(g->out_hdr[c], "samtools",
+                                    "VN", samtools_version(),
+                                    g->arg_list ? "CL": NULL,
+                                    g->arg_list ? g->arg_list : NULL,
+                                    NULL)) {
+        print_error("phase", "failed to add PG line to header");
+        return -1;
+    }
     if (sam_hdr_write(g->out[c], g->out_hdr[c]) < 0) {
         print_error_errno("phase", "Failed to write header for '%s'", g->out_name[c]);
         return -1;
@@ -581,6 +596,7 @@ int main_phase(int argc, char *argv[])
     sam_global_args ga = SAM_GLOBAL_ARGS_INIT;
     static const struct option lopts[] = {
         SAM_OPT_GLOBAL_OPTIONS('-', 0, 0, 0, 0, '-'),
+        {"no-PG", no_argument, NULL, 1},
         { NULL, 0, NULL, 0 }
     };
 
@@ -589,7 +605,7 @@ int main_phase(int argc, char *argv[])
     memset(&g, 0, sizeof(phaseg_t));
     g.flag = FLAG_FIX_CHIMERA;
     g.min_varLOD = 37; g.k = 13; g.min_baseQ = 13; g.max_depth = 256;
-    while ((c = getopt_long(argc, argv, "Q:eFq:k:b:l:D:A:", lopts, NULL)) >= 0) {
+    while ((c = getopt_long(argc, argv, "Q:eFq:k:b:l:D:A", lopts, NULL)) >= 0) {
         switch (c) {
             case 'D': g.max_depth = atoi(optarg); break;
             case 'q': g.min_varLOD = atoi(optarg); break;
@@ -600,6 +616,7 @@ int main_phase(int argc, char *argv[])
             case 'A': g.flag |= FLAG_DROP_AMBI; break;
             case 'b': g.pre = strdup(optarg); break;
             case 'l': fn_list = strdup(optarg); break;
+            case 1: g.no_pg = 1; break;
             default:  if (parse_sam_global_opt(c, optarg, lopts, &ga) == 0) break;
                       /* else fall-through */
             case '?': usage=1; break;
@@ -617,10 +634,11 @@ int main_phase(int argc, char *argv[])
 //      fprintf(stderr, "         -l FILE   list of sites to phase [null]\n");
         fprintf(stderr, "         -F        do not attempt to fix chimeras\n");
         fprintf(stderr, "         -A        drop reads with ambiguous phase\n");
+        fprintf(stderr, "         --no-PG   do not add a PG line\n");
 //      fprintf(stderr, "         -e        do not discover SNPs (effective with -l)\n");
         fprintf(stderr, "\n");
 
-        sam_global_opt_help(stderr, "-....-");
+        sam_global_opt_help(stderr, "-....--.");
 
         return 1;
     }
@@ -635,8 +653,13 @@ int main_phase(int argc, char *argv[])
                 __func__, argv[optind]);
         return 1;
     }
+    if (!g.no_pg && !(g.arg_list = stringify_argv(argc+1, argv-1))) {
+        print_error("phase", "failed to create arg_list");
+        return 1;
+    }
     if (fn_list) { // read the list of sites to phase
         set = loadpos(fn_list, g.fp_hdr);
+        if (set == NULL) return 1;
         free(fn_list);
     } else g.flag &= ~FLAG_LIST_EXCL;
     if (g.pre) { // open BAMs to write
@@ -676,7 +699,7 @@ int main_phase(int argc, char *argv[])
             g.vpos_shift = 0;
             if (lasttid >= 0) {
                 seqs = shrink_hash(seqs);
-                if (phase(&g, g.fp_hdr->target_name[lasttid],
+                if (phase(&g, sam_hdr_tid2name(g.fp_hdr, lasttid),
                           vpos, cns, seqs) < 0) {
                     return 1;
                 }
@@ -748,7 +771,7 @@ int main_phase(int argc, char *argv[])
         }
         if (dophase) {
             seqs = shrink_hash(seqs);
-            if (phase(&g, g.fp_hdr->target_name[tid], vpos, cns, seqs) < 0) {
+            if (phase(&g, sam_hdr_tid2name(g.fp_hdr, tid), vpos, cns, seqs) < 0) {
                 return 1;
             }
             update_vpos(vpos, seqs);
@@ -758,11 +781,11 @@ int main_phase(int argc, char *argv[])
         ++vpos;
     }
     if (tid >= 0) {
-        if (phase(&g, g.fp_hdr->target_name[tid], vpos, cns, seqs) < 0) {
+        if (phase(&g, sam_hdr_tid2name(g.fp_hdr, tid), vpos, cns, seqs) < 0) {
             return 1;
         }
     }
-    bam_hdr_destroy(g.fp_hdr);
+    sam_hdr_destroy(g.fp_hdr);
     bam_plp_destroy(iter);
     sam_close(g.fp);
     kh_destroy(64, seqs);
@@ -778,12 +801,13 @@ int main_phase(int argc, char *argv[])
                         __func__, g.out_name[c]);
                 res = 1;
             }
-            bam_hdr_destroy(g.out_hdr[c]);
+            sam_hdr_destroy(g.out_hdr[c]);
             free(g.out_name[c]);
         }
         free(g.pre); free(g.b);
         if (res) return 1;
     }
+    free(g.arg_list);
     sam_global_args_free(&ga);
     return 0;
 }
